@@ -6,6 +6,7 @@
  * daniel@veillard.com
  */
 
+#define IN_LIBXSLT
 #include "libxslt.h"
 
 #include <string.h>
@@ -20,11 +21,14 @@
 #include "transform.h"
 #include "imports.h"
 #include "keys.h"
+#include "security.h"
 
 #ifdef LIBXML_XINCLUDE_ENABLED
 #include <libxml/xinclude.h>
 #endif
 
+#define WITH_XSLT_DEBUG_DOCUMENTS
+
 #ifdef WITH_XSLT_DEBUG
 #define WITH_XSLT_DEBUG_DOCUMENTS
 #endif
@@ -41,6 +45,8 @@
  * @doc:  a parsed XML document
  *
  * Register a new document, apply key computations
+ *
+ * Returns a handler to the document
  */
 xsltDocumentPtr	
 xsltNewDocument(xsltTransformContextPtr ctxt, xmlDocPtr doc) {
@@ -48,8 +54,7 @@ xsltNewDocument(xsltTransformContextPtr ctxt, xmlDocPtr doc) {
 
     cur = (xsltDocumentPtr) xmlMalloc(sizeof(xsltDocument));
     if (cur == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, (xmlNodePtr) doc);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, (xmlNodePtr) doc,
 		"xsltNewDocument : malloc failed\n");
 	return(NULL);
     }
@@ -69,6 +74,8 @@ xsltNewDocument(xsltTransformContextPtr ctxt, xmlDocPtr doc) {
  * @doc:  a parsed XML document
  *
  * Register a new document, apply key computations
+ *
+ * Returns a handler to the document
  */
 xsltDocumentPtr	
 xsltNewStyleDocument(xsltStylesheetPtr style, xmlDocPtr doc) {
@@ -76,8 +83,7 @@ xsltNewStyleDocument(xsltStylesheetPtr style, xmlDocPtr doc) {
 
     cur = (xsltDocumentPtr) xmlMalloc(sizeof(xsltDocument));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, style, (xmlNodePtr) doc);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, (xmlNodePtr) doc,
 		"xsltNewStyleDocument : malloc failed\n");
 	return(NULL);
     }
@@ -130,6 +136,15 @@ xsltFreeDocuments(xsltTransformContextPtr ctxt) {
 	    xmlFreeDoc(doc->doc);
         xmlFree(doc);
     }
+    cur = ctxt->styleList;
+    while (cur != NULL) {
+	doc = cur;
+	cur = cur->next;
+	xsltFreeDocumentKeys(doc);
+	if (!doc->main)
+	    xmlFreeDoc(doc->doc);
+        xmlFree(doc);
+    }
 }
 
 
@@ -138,7 +153,8 @@ xsltFreeDocuments(xsltTransformContextPtr ctxt) {
  * @ctxt: an XSLT transformation context
  * @URI:  the computed URI of the document
  *
- * Try to load a document within the XSLT transformation context
+ * Try to load a document (not a stylesheet)
+ * within the XSLT transformation context
  *
  * Returns the new xsltDocumentPtr or NULL in case of error
  */
@@ -150,6 +166,21 @@ xsltLoadDocument(xsltTransformContextPtr ctxt, const xmlChar *URI) {
     if ((ctxt == NULL) || (URI == NULL))
 	return(NULL);
 
+    /*
+     * Security framework check
+     */
+    if (ctxt->sec != NULL) {
+	int res;
+	
+	res = xsltCheckRead(ctxt->sec, ctxt, URI);
+	if (res == 0) {
+	    xsltTransformError(ctxt, NULL, NULL,
+		 "xsltLoadDocument: read rights for %s denied\n",
+			     URI);
+	    return(NULL);
+	}
+    }
+
     /*
      * Walk the context list to find the document if preparsed
      */
@@ -161,7 +192,11 @@ xsltLoadDocument(xsltTransformContextPtr ctxt, const xmlChar *URI) {
 	ret = ret->next;
     }
 
+#ifdef XSLT_PARSE_OPTIONS
+    doc = xmlReadFile((const char *) URI, NULL, XSLT_PARSE_OPTIONS);
+#else
     doc = xmlParseFile((const char *) URI);
+#endif
     if (doc == NULL)
 	return(NULL);
 
@@ -169,8 +204,7 @@ xsltLoadDocument(xsltTransformContextPtr ctxt, const xmlChar *URI) {
 #ifdef LIBXML_XINCLUDE_ENABLED
 	xmlXIncludeProcess(doc);
 #else
-	xsltPrintErrorContext(ctxt, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, NULL,
 	    "xsltLoadDocument(%s) : XInclude processing not compiled in\n",
 	                 URI);
 #endif
@@ -180,6 +214,8 @@ xsltLoadDocument(xsltTransformContextPtr ctxt, const xmlChar *URI) {
      */
     if (xsltNeedElemSpaceHandling(ctxt))
 	xsltApplyStripSpaces(ctxt, xmlDocGetRootElement(doc));
+    if (xslDebugStatus == XSLT_DEBUG_NONE)
+	xmlXPathOrderDocElems(doc);
 
     ret = xsltNewDocument(ctxt, doc);
     return(ret);
@@ -190,7 +226,7 @@ xsltLoadDocument(xsltTransformContextPtr ctxt, const xmlChar *URI) {
  * @style: an XSLT style sheet
  * @URI:  the computed URI of the document
  *
- * Try to load a document within the XSLT transformation context
+ * Try to load a stylesheet document within the XSLT transformation context
  *
  * Returns the new xsltDocumentPtr or NULL in case of error
  */
@@ -198,10 +234,27 @@ xsltDocumentPtr
 xsltLoadStyleDocument(xsltStylesheetPtr style, const xmlChar *URI) {
     xsltDocumentPtr ret;
     xmlDocPtr doc;
+    xsltSecurityPrefsPtr sec;
 
     if ((style == NULL) || (URI == NULL))
 	return(NULL);
 
+    /*
+     * Security framework check
+     */
+    sec = xsltGetDefaultSecurityPrefs();
+    if (sec != NULL) {
+	int res;
+
+	res = xsltCheckRead(sec, NULL, URI);
+	if (res == 0) {
+	    xsltTransformError(NULL, NULL, NULL,
+		 "xsltLoadStyleDocument: read rights for %s denied\n",
+			     URI);
+	    return(NULL);
+	}
+    }
+
     /*
      * Walk the context list to find the document if preparsed
      */
@@ -213,7 +266,11 @@ xsltLoadStyleDocument(xsltStylesheetPtr style, const xmlChar *URI) {
 	ret = ret->next;
     }
 
+#ifdef XSLT_PARSE_OPTIONS
+    doc = xmlReadFile((const char *) URI, NULL, XSLT_PARSE_OPTIONS);
+#else
     doc = xmlParseFile((const char *) URI);
+#endif
     if (doc == NULL)
 	return(NULL);
 
@@ -224,7 +281,7 @@ xsltLoadStyleDocument(xsltStylesheetPtr style, const xmlChar *URI) {
 /**
  * xsltFindDocument:
  * @ctxt: an XSLT transformation context
- * @@doc: a parsed XML document
+ * @doc: a parsed XML document
  *
  * Try to find a document within the XSLT transformation context
  *
