@@ -10,6 +10,7 @@
 #define __XML_XSLT_PRECOMP_H__
 
 #include <libxml/tree.h>
+#include "xsltexports.h"
 #include "xsltInternals.h"
 
 #ifdef __cplusplus
@@ -21,13 +22,17 @@ extern "C" {
  */
 extern const xmlChar *xsltExtMarker;
 
-xsltElemPreCompPtr xsltDocumentComp	(xsltStylesheetPtr style,
+XSLTPUBFUN xsltElemPreCompPtr XSLTCALL 
+		xsltDocumentComp	(xsltStylesheetPtr style,
 					 xmlNodePtr inst,
 					 xsltTransformFunction function);
 
-void		xsltStylePreCompute	(xsltStylesheetPtr style,
+XSLTPUBFUN void XSLTCALL		
+		xsltStylePreCompute	(xsltStylesheetPtr style,
 					 xmlNodePtr inst);
-void		xsltFreeStylePreComps	(xsltStylesheetPtr style);
+XSLTPUBFUN void XSLTCALL		
+		xsltFreeStylePreComps	(xsltStylesheetPtr style);
+
 #ifdef __cplusplus
 }
 #endif
